// -*- C++ -*-
//
// GSI version by WHS using the original sound.cpp & Maelstrom_Sound.cpp.
//

#include <sys/types.h>
#include <stdlib.h>
//#include <sys/time.h>
//#include <sys/socket.h>
//#include <sys/file.h>
#include <stdio.h>
#include <signal.h>
//#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "mydebug.h"

#include "sound.h"
#include "soundres.h"
#include <gsi/gsi.h>


extern char  *file2libpath(char *filename);	// From shared.cc
static SoundRes sounds(file2libpath("Maelstrom Sounds"));


SoundClient:: SoundClient(void)
{
	int   i;

#ifdef USE_GSI
	if (gsi_init(NULL)) {
		fprintf(stderr, "GSI: Unable to connect to server\n");
		//return -1;
		exit(255);
	}
	gsi_init_pcm(11025, 1, GSI_8BIT, 8);
	gsi_enable_events(GSI_EVMASK_CHANNEL);
#endif

	/* Initialize the channels */
	for ( i=0; i<NUM_CHANNELS; ++i ) {
		channels[i].in_use = 0;
		channels[i].Callback = NULL;
	}
	Callback_Q.next = NULL;
	Callback_Qtail = &Callback_Q;
	awaiting_reply = 0;
}

SoundClient:: ~SoundClient()
{
	gsi_close();
}


void
SoundClient:: CallBack(unsigned short channel)
{
	channels[channel].in_use = 0;

	if ( awaiting_reply > 0 )
		QueueCallback(channel);
	else {
		if ( channels[channel].Callback )
			channels[channel].Callback(channel);
	}
}


int
SoundClient:: SetVolume(unsigned short vol)
{
	gsi_set_volume(GSI_PCM, vol * 32);
	return 0;
}

int
SoundClient:: LoadSound(short sndID)
{
	Sample *sample;

	if (sounds.LoadSound(sndID) < 0) {
		error("Sound: Invalid sound id: %hd\n", sndID);
	}
	//gsi_load_sample_from (sndID-100, "Maelstrom Sounds", offset);

	sample = sounds.GetSound(sndID);
	gsi_send_raw_sample (sndID-100, 11025, 1, GSI_8BIT, sample->len, sample->data);
	sounds.FreeSound(sndID);

	return 0;
}

int
SoundClient:: PlaySound(short sndID, short priority, 
					void (*STCallBack)(unsigned short))
{
	unsigned short channel;

	for ( channel=0; channel<NUM_CHANNELS; ++channel) {
		if ( priority > Status(channel) )
			break;
	}
	if ( channel == NUM_CHANNELS )
		return(-1);

	return(PlayChannel(sndID, priority, channel, STCallBack));
}

void
SoundClient:: channel_cb(int ch) {
//	if (gsi_get_channel_status(ch) == 0) channels[ch].in_use = 0;
//	if (channels[ch].Callback)
//		channels[ch].Callback(ch);
	if (gsi_event_data[0]==GSI_EV_CHANNEL_END) {
		CallBack(ch);
	}
}
extern void gsi_channel_handler(int ch);

int
SoundClient:: PlayChannel(short sndID, short priority, 
		unsigned short theChannel, void (*STCallBack)(unsigned short))
{
	if ( priority <= Status(theChannel) )
		return(-1);

	channels[theChannel].in_use = 1;
	channels[theChannel].ID = sndID;
	channels[theChannel].priority = priority;
	channels[theChannel].Callback = STCallBack;

//	gsi_set_channel_callback(theChannel, (void *) channel_cb);
	gsi_set_channel_callback(theChannel, gsi_channel_handler);
	gsi_play_sound(theChannel, sndID-100, GSI_CMD_OVERWRITE);
	return theChannel;
}

void
SoundClient:: HaltSounds(void)
{
	unsigned short channel;

	gsi_kill_sounds ();

	for ( channel=0; channel<NUM_CHANNELS; ++channel )
		channels[channel].in_use = 0;
}

void
SoundClient:: HaltChannel(unsigned short channel)
{
	gsi_kill_sound (channel);

	channels[channel].in_use = 0;
}


int
SoundClient:: SoundID(unsigned short channel)
{
	if ( channels[channel].in_use )
		return(channels[channel].ID);

	return(-1);
}

int
SoundClient:: IsSoundPlaying(short sndID)
{
	unsigned short channel;

	gsi_flush(); // for channel callback (lots of IsSoundPlaying loops in fastlogic/game.cpp & netlogic/game.cpp
	for ( channel=0; channel<NUM_CHANNELS; ++channel) {
		if ( sndID ) {
			if ( sndID == SoundID(channel) ) {
				return(1);
			}
		} else {
			if ( SoundID(channel) != -1 ) {
				return(1);
			}
		}
	}
	return(0);
}


int
SoundClient:: Status(unsigned short channel)
{
	if ( channels[channel].in_use )
		return(channels[channel].priority);

	return(-1);
}

void
SoundClient:: ChannelStatus(unsigned short *chanOnePriority,
			    unsigned short *chanTwoPriority,
			    unsigned short *chanThreePriority,
			    unsigned short *chanFourPriority)
{
	*chanOnePriority = Status(CHANNEL_ONE);
	*chanTwoPriority = Status(CHANNEL_TWO);
	*chanThreePriority = Status(CHANNEL_THREE);
	*chanFourPriority = Status(CHANNEL_FOUR);
}
